#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>
#include "graph_lib.h"

#define LOG_DEBUG 1
#define LOG_WARN 2
#define LOG_ERR 3
#define LOG_LEVEL LOG_WARN


/* swaps content of two memory places */
void swap( void * vp1, void * vp2, int size ) {
	char buffer[size];
	memcpy( buffer, vp1, size );
	memcpy( vp1, vp2, size );
	memcpy( vp2, buffer, size );
}


/*


typedef struct edge_t edge_t;
typedef struct edge_iterator_t edge_iterator_t;
typedef struct graph_t graph_t;
typedef struct coloring_t coloring_t;
typedef struct code_properties_t code_properties_t;
typedef long nv_t; //max number of vertices type
typedef int unsigned ne_t; //max number of edges type
typedef short color_t;

struct edge_t {
	nv_t vertex;
	int unsigned data;
	edge_t * next;
} edge;


struct graph_t {
	int num_vertices;
	edge_t * vertices;
};

struct edge_iterator_t {
	graph_t *graph;
	edge_t *edge;
	nv_t vertex;
};

struct coloring_t {
	nv_t num_vertices;
	char num_colors;
	color_t *colors;
};


struct code_properties_t {
	nv_t num_vertices;
	char num_colors;
	short num_bits, num_bytes;	
};

char colors[7][7] = {
	"FF0000",
	"00AA00",
	"0000FF",
	"000000",
	"da70d6",
	"AA00AA",
	"00AAAA"
};
*/

void msg(int type, const char* format, ...) {
	if (type<LOG_LEVEL) return;
	
	switch (type)
	{
		case LOG_DEBUG:
			fprintf( stderr, "DEBUG: " );
			break;
		case LOG_WARN:
			fprintf( stderr, "WARNING: " );
			break;
		case LOG_ERR:
			fprintf( stderr, "ERROR: " );
			break;
	}
	
	va_list args;
    va_start( args, format );	
	vfprintf( stderr, format, args );	
    va_end( args );
    fprintf( stderr, "\n" );
    fflush( stderr );
}



// ******************   GRAPH   ******************



graph_t *g_create(int num_vertices )  {
	graph_t *graph;
	graph = malloc( sizeof( graph_t) );

	//set number of vertices
	graph->num_vertices = num_vertices;
	
	//allocate vertices for the graph
	edge_t * vertices = calloc( num_vertices, sizeof( edge_t ) );
	
	int i;
	for ( i=0; i<num_vertices; i++ ) {
		vertices[i].vertex = -1;
		vertices[i].next = NULL;
	}
	
	graph->vertices = vertices;
	return graph;
}



int g_num_vertices( graph_t * graph ) {
	return graph->num_vertices;
}




/* adds vertex v2 to the list of incident vertices of vertex v1.
returns 1 on success, 0 otherwise */
int g_add_halfedge( graph_t * graph, int v1, int v2 ) {
	edge_t * edge = &(graph->vertices[v1]);
	//if this is the first edge incident with the vertex v1
	if ( edge->vertex < 0 ) {
		edge->vertex = v2;
		return 1;
	}
	
	if( edge->vertex == v2 ) {
//		msg( LOG_WARN, "you tried to add an edge that already exists" );
		return 0;
	}

	while( (*edge).next!=NULL ) {
		edge = edge->next;
		if( edge->vertex == v2 ) {
//			msg( LOG_WARN, "you tried to add an edge that already exists" );
			return 0;
		}
	}
	edge->next = malloc( sizeof( edge_t ) );
	edge = edge->next;
	edge->vertex = v2;
	edge->next = NULL;
	return 1;
}



void g_add_edge( graph_t * graph, nv_t v1, nv_t v2 ) {
	if (v1<0 || v2<0 || v1>=graph->num_vertices || v2>=graph->num_vertices) {
		msg( LOG_ERR, "you're trying to create out-of boundary edge [%d, %d]", v1, v2 );	
	}
	
	//perform second addition only if the first was successful
	if ( g_add_halfedge( graph, v1, v2 ) ) {
		g_add_halfedge( graph, v2, v1 ); 
	}
}	


int g_remove_halfedge( graph_t * graph, int v1, int v2 ) {
	edge_t * edge = &(graph->vertices[v1]);
	edge_t * previous_edge = NULL;
	
	//if there is no edge incident with the vertex v1
	if ( edge->vertex < 0 ) {
		msg( LOG_WARN, "trying to remove non-existing edge [%d, %d]. Nothing happened", v1, v2 );
		return 0;
	}

	while( edge->next!=NULL && edge->vertex != v2 ) {
		previous_edge = edge;
		edge = edge->next;
	}

	if ( (*edge).vertex == v2 ) {
		//ok, let's delete this edge
		if (previous_edge == NULL) {
			if ( edge->next == NULL ) {
				edge->vertex = -1;
			} else {
				edge_t * next = edge->next;
				edge->vertex = next->vertex;
				edge->next = next->next;
				free( next );
			}
		} else {
			previous_edge->next = edge->next;
			free( edge );
		}
		return 1;
	} else {
		msg( LOG_WARN, "trying to remove non-existing edge [%d, %d]. Nothing happened", v1, v2 );
		return 0;
		
	}
}


void g_remove_edge( graph_t * graph, int v1, int v2 ) {
	//perform second deletion only if the first was successful
	if ( g_remove_halfedge( graph, v1, v2 ) ) {
		g_remove_halfedge( graph, v2, v1 ); 
	}
}	



int g_is_edge( graph_t * graph, int v1, int v2 ) {
	edge_t * edge = &(graph->vertices[v1]);
	if ( edge->vertex < 0 ) {
		return 0;
	} else {
		if( edge->vertex == v2 ) return 1;
		while( edge->next!=NULL ) {
			edge = edge->next;
			if( edge->vertex == v2 ) return 1;
		}
		return 0;
	}
}


void g_dump( graph_t * graph ) {
	printf("Number of vertices: %d", graph->num_vertices);
	nv_t i;
	
	struct edge_t edge;
	for ( i=0; i<graph->num_vertices; i++ ) {
		printf( "\n" );
		printf( "%3ld: ", i );

		edge = graph->vertices[i];
		if ( edge.vertex<0 ) continue;
		
		printf( "%3ld", edge.vertex );
		while ( edge.next!=NULL ) {
			edge = *(edge.next);
			printf( ", %3ld", edge.vertex );
		}				
	}
	printf( "\n" );

}

void g_dump_tgf( graph_t * graph ) {
	nv_t i;
	for ( i=0; i<graph->num_vertices; i++ ) {
		printf( "%ld %ld \n", i, i );
	}
	printf( "#\n" );

	struct edge_t edge;
	for ( i=0; i<graph->num_vertices; i++ ) {
		edge = graph->vertices[i];
		if ( edge.vertex<0 ) continue;
		
		if (i<edge.vertex)
			printf( "%ld %ld\n", i, edge.vertex );
		while ( edge.next!=NULL ) {
			edge = *(edge.next);
			if (i<edge.vertex)
				printf( "%ld %ld\n", i, edge.vertex );
		}				
	}
	printf( "\n" );

}

void g_dump_graphml( graph_t * graph ) {

	//print header
	printf( "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" );
	printf( "<graphml xmlns=\"http://graphml.graphdrawing.org/xmlns\"\n" );
	printf( "         xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n" );  
	printf( "         xsi:schemaLocation=\"http://graphml.graphdrawing.org/xmlns\n" );  
	printf( "         http://www.yworks.com/xml/schema/graphml/1.1/ygraphml.xsd\"\n" );  
	printf( "         xmlns:y=\"http://www.yworks.com/xml/graphml\">\n" );  
	printf( "  <key id=\"d0\" for=\"node\" yfiles.type=\"nodegraphics\"/>  \n" );
	printf( "  <key id=\"d1\" for=\"edge\" yfiles.type=\"edgegraphics\"/>  \n" );
	printf( "  <graph id=\"G\" edgedefault=\"undirected\"> \n " );

	
	//print nodes
	nv_t i;
	for ( i=0; i<graph->num_vertices; i++ ) {
		printf("    <node id=\"n%lu\">\n", i );
		printf("      <data key=\"d0\">\n" );
		printf("        <y:ShapeNode>\n" );
//		printf("          <y:Geometry width=\"10.0\" height=\"10.0\"/>\n" );
		printf("          <y:Fill color=\"#EEEEEE\" transparent=\"false\"/>\n" );
  
		printf("          <y:NodeLabel>%lu</y:NodeLabel>\n", i );
		printf("          <y:Shape type=\"ellipse\"/>\n" );
		printf("        </y:ShapeNode>\n" );
		printf("      </data>\n" );
		printf("    </node>\n\n" );
	}

	//print edges
	struct edge_t edge;
	for ( i=0; i<graph->num_vertices; i++ ) {
		edge = graph->vertices[i];
		if ( edge.vertex<0 ) continue;
		
		while( 1 ) {
			//print edge [a,b] only if a<b, so that there are not duplicate edges
			if (i<edge.vertex) {
    printf("    <edge id=\"e%lu%lu\" source=\"n%lu\" target=\"n%lu\">\n", i, edge.vertex, i, edge.vertex);
    printf("      <data key=\"d1\">                                            \n");
    printf("    	<y:PolyLineEdge>                                         \n");
    printf("    	  <y:LineStyle type=\"line\" width=\"3.0\" color=\"#000000\"/> \n"); 
    printf("    	  <y:EdgeLabel></y:EdgeLabel>                            \n");
    printf("    	</y:PolyLineEdge>                                        \n");
    printf("      </data>                                                    \n");
    printf("    </edge>                                                      \n");
			}
			if (edge.next==NULL) break;
			edge = *(edge.next);
		};
		
	}
	printf( "\n" );

}


//	(* graph).vertices[v2].vertex = v1;

void g_destroy(graph_t ** graph) {
	nv_t i;
	edge_t *current, *next;
	for ( i=0; i<(*graph)->num_vertices; i++ ) {
		current = &((*graph)->vertices[i]);
		if (current->next == NULL) continue;
		
		current = current->next;
		while ( 1 ) {
			next = current->next;
			free( current );
			if( next == NULL) break;
			current = next;			
		}
	}
	
	free( (*graph)->vertices );
	free( *graph );
	*graph = NULL;
}


void g_set_data( graph_t *graph, int unsigned data, nv_t v1, nv_t v2 ) {
	if (v1<0 || v2<0 || v1 >= graph->num_vertices || v2 >= graph->num_vertices) {
		msg( LOG_ERR, "out of bounds: no such edge %d %d", v1, v2 );
		exit(1);
	}

	edge_t *edge = &(graph->vertices[v1]);
	
	while( edge->vertex!=v2 && edge->next!=NULL ) {
		edge = edge->next;
	}
	
	if (edge->vertex == v2) {
		edge->data = data;
		return;
	}
	
	msg( LOG_ERR, "no such edge %d %d", v1, v2 );
	return;
}

int g_get_data( graph_t *graph, nv_t v1, nv_t v2 ) {
	if (v1<0 || v2<0 || v1 >= graph->num_vertices || v2 >= graph->num_vertices) {
		msg( LOG_ERR, "out of bounds: no such edge %d %d", v1, v2 );
		exit(1);
	}

	edge_t *edge = &(graph->vertices[v1]);
	
	while( edge->vertex!=v2 && edge->next!=NULL ) {
		edge = edge->next;
	}
	
	if (edge->vertex == v2) {
		return edge->data;
	}
	
	msg( LOG_ERR, "no such edge %d %d", v1, v2 );
	return -1;
}


void g_set_data_both_directions( graph_t *graph, int unsigned data, nv_t v1, nv_t v2 ) {
	g_set_data( graph, data, v1, v2 );
	g_set_data( graph, data, v2, v1 );
	return;	
}
	
/* iterates through all edges and stores their number to edge's data.
returns total number of edges. */
ne_t g_enumerate_edges( graph_t *graph ) {
	nv_t i;
	ne_t ne = 0;
	
	edge_t *edge;
	
	for ( i = 0; i < graph->num_vertices; i++ ) { //for all vertices
		edge = &(graph->vertices[i]);
		if (edge->vertex<0) continue;

		while( 1 ) { 
			if ( edge->vertex > i ) {
				edge->data = ne;
				g_set_data( graph, ne, edge->vertex, i );
				ne++;
			}
			if ( edge->next == NULL ) break;
			edge = edge->next;
		}
	}
	return ne;
}


void g_clean_data( graph_t *graph ) {
	g_set_data_everywhere( graph, 0 );
	return;
}

void g_set_data_everywhere( graph_t *graph, int new_data ) {
	nv_t i;
	edge_t *edge;
	
	for ( i=0; i<graph->num_vertices; i++ ) { //for all vertices
		edge = &(graph->vertices[i]);

		while( 1 ) { //erase data from all incident edges
			edge->data = new_data;
			if (edge->next==NULL) break;
			edge = edge->next;
		}
	}
	return;
}



graph_t *g_create_linegraph_from_enumerated_graph( graph_t * graph, ne_t ne ) {
	nv_t i = 0;
	edge_t *edge, *edge2;

	graph_t *linegraph;
	linegraph = g_create( ne );
	
	for ( i=0; i<graph->num_vertices; i++ ) {
		//at most one edge is incidence with current vertex, so
		//no edge is added to linegraph
		if ( graph->vertices[i].next == NULL ) continue;
		
		edge = &(graph->vertices[i]);

		while ( edge->next != NULL ) {

			edge2 = edge->next;
			
			while ( edge2 != NULL ) {
				g_add_edge( linegraph, edge->data, edge2->data );
				edge2 = edge2->next;
			}
			edge = edge->next;
		}
	}
	
	return linegraph;
}


graph_t *g_create_linegraph( graph_t * graph ) {
	ne_t ne = 0;
	ne = g_enumerate_edges( graph );
	graph_t *linegraph = g_create_linegraph_from_enumerated_graph( graph, ne );
	return linegraph;
}


graph_t *g_duplicate( graph_t *original) {
	return( g_duplicate_and_extend( original, 0 ) );
}

graph_t *g_duplicate_and_extend( graph_t *original, int num_extending_vertices) {
	graph_t *graph;
	graph = g_create( original->num_vertices+num_extending_vertices );

	nv_t i, j;
	for ( i = 0; i < original->num_vertices-1; i++ ) {
		for ( j = i+1; j < original->num_vertices; j++ ) {
			if (g_is_edge( original, i, j) ) {
				g_add_edge( graph, i, j );
			}
		}
	}
	return( graph );
}




// ******************   COLORING   ******************



coloring_t *g_create_coloring( char num_colors, nv_t nv ) {
	coloring_t *coloring = malloc( sizeof( coloring_t ) );
	coloring->colors = malloc( nv * sizeof( color_t ) );	
	coloring->num_colors = num_colors;	
	coloring->num_vertices = nv;	

	int i;
	for ( i=0; i<nv; i++ )
		coloring->colors[i] = -1;

	return coloring;
}

void g_destroy_coloring( coloring_t **coloring ) {
	free( (*coloring)->colors );
	free( (*coloring) );
}


void g_coloring_copy_colors( coloring_t *original,  coloring_t *new ) {
	int i;
	for ( i=0; i<original->num_vertices; i++ )
		new->colors[i] = original->colors[i];
	return;
}


edge_t *g_edge_iterator_next( edge_iterator_t *iterator ) {
	
	if (iterator->vertex > -1 && iterator->edge == NULL) {
		return NULL;
	}
	//iterator hasn't return any edge yet or last returned edge was last neighbor
	if ( iterator->vertex == -1 || iterator->edge->next == NULL) {
		iterator->vertex++;
		while (iterator->vertex < iterator->graph->num_vertices) {
			iterator->edge = &(iterator->graph->vertices[iterator->vertex]);
			if ( iterator->edge->vertex >= 0 ) {
				return iterator->edge;
			}
			iterator->vertex++;
		}
		//no more edges
		return NULL;		
	} else {
		iterator->edge = iterator->edge->next;
		return iterator->edge;
	}
}

edge_t *g_edge_iterator_next_for_vertex( edge_iterator_t *iterator ) {
	if ( iterator->vertex < 0 ) {
		msg( LOG_ERR, "g_edge_iterator_next_for_vertex: vertex is not set" );
		return NULL;
	}
	
	if (iterator->edge == NULL) {
		iterator->edge = &(iterator->graph->vertices[iterator->vertex]);
		if (iterator->edge->vertex >= 0)
			return iterator->edge;
	}

	if (iterator->edge->vertex<0)
		return NULL;

	//iterator hasn't return any edge yet or last returned edge was last neighbor
	if ( iterator->edge->next == NULL ) {
		return NULL;		
	} else {
		iterator->edge = iterator->edge->next;
		return iterator->edge;
	}
}


edge_iterator_t g_new_edge_iterator(graph_t *graph) {
	edge_iterator_t iterator;
	iterator.graph = graph;
	iterator.vertex = -1;
	iterator.edge = NULL;
	return iterator;
}


short g_is_vertex_colored_regularly( graph_t *graph, color_t *color, nv_t current_vertex ) {

	edge_iterator_t iterator = g_new_edge_iterator( graph );
	iterator.vertex = current_vertex;
	edge_t *edge;
	color_t c = color[current_vertex];
	if (c == -1) return 0;
	
	while ( NULL != (edge = g_edge_iterator_next_for_vertex( &iterator ) ) ) {
		if ( edge->vertex<current_vertex && color[edge->vertex] == c ) {
			return 0;
		}
	}
	return 1;	
}

short g_is_colored_regularly( graph_t *graph, color_t *color ) {
	nv_t i;
	for ( i = 0; i < graph->num_vertices; i++ ) {
		if (0 == g_is_vertex_colored_regularly( graph, color, i ) ) {
			return 0;
		}
	}
	return 1;
}


char g_get_next_coloring_offset( graph_t *graph, coloring_t *coloring, nv_t offset ) {
	if ( offset > graph->num_vertices ) {
		msg( LOG_ERR, "offset > number of vertices" );
		exit(0);
	} else if ( offset == graph->num_vertices ) {
		msg( LOG_WARN, "offset == number of vertices" );
		return 2;
	}
	
	color_t *color = coloring->colors;

	nv_t nv = graph->num_vertices;
	nv_t current_vertex = offset;
	
	color[nv-1]++;
	
	while (1) {
		
		if (color[current_vertex] >= coloring->num_colors) {
			color[current_vertex] = -1;
			current_vertex--;
			if (current_vertex < offset) {
				msg( LOG_DEBUG, "impossible to color" );
				return 0;
			}
			color[current_vertex]++;
			continue;
		}
		
		if (g_is_vertex_colored_regularly( graph, color, current_vertex )) {
			current_vertex++;
		} else {
			color[current_vertex]++;
		}

		if ( current_vertex == graph->num_vertices ) {
				msg( LOG_DEBUG, "we've got a coloring" );
				return 1;
		}
	}
}


char g_get_next_coloring( graph_t *graph, coloring_t *coloring ) {
	return g_get_next_coloring_offset( graph, coloring, 0 );
}



void g_erase_coloring( coloring_t *coloring ) {
	int i;
	for (i=0; i<coloring->num_vertices; i++) {
		coloring->colors[i] = -1;
	}
}

void g_init_coloring_for_regular_graph( coloring_t *coloring, int degree ) {
	g_erase_coloring( coloring );
	int i;
	for (i=0; i<degree; i++) {
		coloring->colors[i] = i;
	}

};


void g_dump_coloring( coloring_t *coloring ) {
	nv_t i;
	for ( i=0; i<coloring->num_vertices; i++ ) {
		printf( "%3d  ", coloring->colors[i] );
		if (i%30==29) printf("\n");
	}
	printf( "\n" );
}


//this assumes that graph's vertices are properly enumerated
//using g_enumerate_edges method
color_t g_get_edge_color( graph_t *graph, coloring_t *linegraph_coloring,
	nv_t vertex1, nv_t vertex2 ) {

	int unsigned edge_number = g_get_data( graph, vertex1, vertex2 ); 
	return linegraph_coloring->colors[edge_number];
}


void g_dump_dot_edgecoloring( graph_t *graph, coloring_t *linegraph_coloring ) {
	nv_t i, j;
	printf( "graph edgecoloring {\n" );
	for ( i=0; i<graph->num_vertices-1; i++ ) {
		for ( j=i+1; j<graph->num_vertices; j++ ) {
			
			if ( g_is_edge( graph, i, j ) ) {
				printf( "   %lu -- %lu [label=\"%d\"];\n", i, j, 
					g_get_edge_color(  graph, linegraph_coloring, i, j ) );
			}
		}
	}
	printf( "}\n\n" );
}

void g_dump_tgf_edgecoloring( graph_t *graph, coloring_t *linegraph_coloring ) {
	nv_t i, j;
	for ( i=0; i<graph->num_vertices; i++ ) {
				printf( "%lu %lu\n", i, i );
	}

	printf( "#\n");

	for ( i=0; i<graph->num_vertices-1; i++ ) {
		for ( j=i+1; j<graph->num_vertices; j++ ) {
			
			if ( g_is_edge( graph, i, j ) ) {
//				printf( "%lu %lu\n", i, j );
				printf( "%lu %lu %d:%d\n", i, j, 
					g_get_data( graph, i, j ) ,g_get_edge_color(  graph, linegraph_coloring, i, j ) );
			}
		}
	}
}

void g_dump_tgf_coloring( graph_t *graph, coloring_t *coloring ) {
	nv_t i, j;
	for ( i=0; i<graph->num_vertices; i++ ) {
				printf( "%lu %lu-%d\n", i, i, coloring->colors[i] );
	}
	printf( "#\n");
	for ( i=0; i<graph->num_vertices-1; i++ ) {
		for ( j=i+1; j<graph->num_vertices; j++ ) {
			if ( g_is_edge( graph, i, j ) ) {
				printf( "%lu %lu\n", i, j );
			}
		}
	}
}


//this function assumes that graph's vertices are properly enumerated
//using g_enumerate_edges method and graph is
//regular with degree equivalent to number of colors in coloring
int g_get_vertex_sign( graph_t *graph, coloring_t *linegraph_coloring,
	nv_t vertex ) {

	int sign;
	short degree = linegraph_coloring->num_colors;
	short colors[degree], used[degree], i;
	
	//initialize permutation array (colors) and auxiliary array "used"
	edge_t *edge = &(graph->vertices[vertex]);
	for ( i = 0; i<degree; i++ ) {
		used[i] = 0;
		colors[i] = linegraph_coloring->colors[edge->data];
		edge = edge->next;	
	}

	short current;
	sign = 1;
	
	//compute parity:
	//find cycles, update parity according to the length of the cycles
	for ( i = 0; i<degree; i++ ) {
		//don't use any cycle more than once
		if (used[i]) continue;
		
		used[i]=1;
		current = colors[i];
		while( !used[current] ) {
			used[current] = 1;
			current = colors[current];
			sign *= -1;
		}
	}
	
	return sign;
}


color_t g_get_graph_sign( graph_t *graph, coloring_t *linegraph_coloring ) {
	nv_t i;
	int sign = 1, vertex_sign;
	for ( i = 0; i<graph->num_vertices; i++ ) {
		vertex_sign = g_get_vertex_sign( graph, linegraph_coloring, i );
		//printf( "vertex sign: %d\n", vertex_sign );

	
		sign *= vertex_sign;
	}
	return sign;
}





char *get_color_code( int color, char *buffer ) {
	char color_codes[7][7] = {
		"FF0000",
		"00AA00",
		"0000FF",
		"000000",
		"ffcc00",
		"AA00AA",
		"00AAAA"
	};
	sprintf( buffer, "%s", color_codes[color] );
	return buffer;
}





// ******************   CODE   ******************




short g_num_bits( short num_colors ) {
	if ( num_colors <= 2 )
		return 1;
	else if ( num_colors <= 4 )
		return 2;
	else
		return ceil( log( num_colors ) / log( 2 ) );
}


code_properties_t g_code_properties( short num_colors, nv_t num_vertices ) {
	code_properties_t properties;
	properties.num_colors = num_colors;
	properties.num_vertices = num_vertices;
	properties.num_bits = g_num_bits( num_colors );
	properties.num_bytes = (short) ceil( properties.num_bits * num_vertices / 8.0 );
	return properties;
}



char *g_create_code( code_properties_t *properties ) {
	return malloc( sizeof( char ) * properties->num_bytes );
}


void g_destroy_code( char **code ) {
	free( *code );
}


void g_erase_code( char *code, code_properties_t *properties ) {
	int i;
	for (i=0; i<properties->num_bytes; i++) {
		code[i] = 0;
	}
}



void g_coloring_to_code( coloring_t *coloring, char *code, code_properties_t *properties ) {
	if ( ( coloring->num_vertices != properties->num_vertices ) ||
		( coloring->num_colors != properties->num_colors ) ) {
		msg( LOG_ERR, "coloring and properties are not compatible!" );
	}

	
	g_erase_code( code, properties );
	
	short j, current_bit = 0, current_byte = 0;
	nv_t i;
	char c;
	for( i = 0; i<coloring->num_vertices; i++ ) {
		
		c = coloring->colors[i];
		for( j = 0; j<properties->num_bits; j++ ) {
			code[current_byte] <<= 1;
			code[current_byte] |= ( ( c & (1 << (properties->num_bits-1) ) ) >> (properties->num_bits-1) );
			c <<= 1;
			current_bit++;
			
			if( current_bit == 8 ) {
				current_bit = 0;
				current_byte++;
			}
		}
	}
	return;
}

void g_code_to_coloring( char *code, coloring_t *coloring, code_properties_t *properties ) {
	if ( ( coloring->num_vertices != properties->num_vertices ) ||
		( coloring->num_colors != properties->num_colors ) ) {
		msg( LOG_ERR, "coloring and code are not compatible!" );
	}

	msg(LOG_DEBUG, "code to coloring ");
	
	nv_t i, nv = coloring->num_vertices;
	short j;
	short current_bit = 0, current_byte = 0;
	short offset = properties->num_bytes * 8 - nv * properties->num_bits;
	
	char c;
	c = code[current_byte];
	if (current_byte == properties->num_bytes-1) {
		c <<= offset;
	}
	
	for( i = 0; i < nv; i++ ) {
		coloring->colors[i] = 0;
		for( j = 0; j<properties->num_bits; j++ ) {
			coloring->colors[i] <<= 1;
			coloring->colors[i] |= ( (c & ( 1 << 7 ) ) >> 7);
			c <<= 1;
			current_bit++;
			if( current_bit==8 ) {
				current_bit = 0;
				current_byte++;
				c = code[current_byte];
				if (current_byte == properties->num_bytes-1) {
					c <<= offset;
				}
			}
		}
	}
	return;
}


char g_code_compare( char *code1, char *code2, code_properties_t *properties ) {
	int i;
	
	for ( i = 0; i < properties->num_bytes; i++ ) {
		if ( (unsigned char)code1[i] < (unsigned char)code2[i] ) {
			return (-1);
		}
		
		if ( (unsigned char)code1[i] > (unsigned char)code2[i] ) {
			return (1);
		}
//		printf( "%d %d\n", (unsigned char)code1[i], (unsigned char)code2[i] );
		
	}
	return 0;
}


void g_dump_code( char *code, code_properties_t *properties ) {
	int i;
	for ( i=0; i<properties->num_bytes; i++ ) {
		printf( "%d ", (unsigned char)code[i]);
	}
	printf( "\n" );
}


// ******************   SIGN (of coloring)   ******************
/* 
void g_get_vertex_sign( nv_t vertex, graph_t *graph, coloring_t *coloring ) {
	edge_iterator_t iterator = g_new_edge_iterator( graph );
	iterator.vertex = vertex;
	edge_t *edge;
	color_t c = color[current_vertex];

	if (c == -1) return 0;
	
	while ( NULL != (edge = g_edge_iterator_next_for_vertex( &iterator ) ) ) {
		if ( edge->vertex<current_vertex && color[edge->vertex] == c ) {
			return 0;
		}
	}
	return 1;	

}

	*/




// ******************   DICTIONARY   ******************


char **g_create_dictionary( unsigned long num_items ) {
	return malloc( num_items * sizeof( char * ) ); 
}

void g_destroy_dictionary(char ***dictionary) {
	free(*dictionary);
}

long g_dictionary_get_index(char **dictionary, char *item, code_properties_t *code_properties, long offset, long num_items) {
/*
	printf( "looking for: " ); g_dump_code( item, code_properties );
	printf(" offset: %ld,  num_items: %ld \n", offset, num_items );
	printf( "checking (index %ld) :", offset+num_items/2 ); g_dump_code( dictionary[offset + num_items/2], code_properties );
	printf( "\n" );

	*/
	
//binary search for the index
	if( num_items<50 ) {
		long i;
		for (i=0; i<num_items; i++) {
			if (g_code_compare( item, dictionary[offset+i], code_properties ) == 0) {
				return offset+i;
			};
		}
		msg( LOG_ERR, "edge hasn't been found in the dictionary %s", item );
		return -1;
	} else {
		char result = g_code_compare( item, dictionary[offset + num_items/2], code_properties );
		if (result == 0 )
			return offset + num_items/2;
		else if (result < 0) {
			return g_dictionary_get_index(dictionary, item, code_properties, offset, num_items/2);	
		} else {
			return g_dictionary_get_index(dictionary, item, code_properties, offset+num_items/2, ceil(num_items/2.0));	
		}
	}
}

// ******************   KEMPE   ******************



void g_breadth_first_search( graph_t *graph, coloring_t *coloring, nv_t starting_vertex, color_t new_color, nv_t *mark ) {
	nv_t current_vertex;
	edge_iterator_t iterator;
	edge_t *edge;
	color_t color1 = new_color, color2 = coloring->colors[starting_vertex];

	nv_t queue[coloring->num_vertices], i;
	for( i=0; i<coloring->num_vertices; i++ ) {
		queue[i] = -1;
		mark[i] = 0;
	}


	//put first vertex to the queue
	nv_t position=0;
	nv_t num_queued=0;
	queue[position+num_queued] = starting_vertex;
	num_queued++;
	
	while ( num_queued>0 ) {
		
		current_vertex = queue[position]; //pop vertex from queue
		num_queued--;
		position++;
		
		mark[current_vertex] = 1; //mark it for change
		
		iterator = g_new_edge_iterator(graph); //get all neighbors of current vertex
		iterator.vertex = current_vertex;
	
		//add to queue - each neighbor not yet marked
		while ( NULL!= (edge = g_edge_iterator_next_for_vertex( &iterator ) ) ) {
			if (mark[edge->vertex]==0 && ( coloring->colors[edge->vertex]==color1 || coloring->colors[edge->vertex]==color2 ) ) { 
				//add vertex to queue
				queue[position+num_queued] = edge->vertex;
				num_queued++;
			} else {
				//vertex is already marked for switch or wrong color
				// so we don't do anything 
			}
		}
	}
	
}



//changes coloring, updates to_do
void g_kempe_switch( coloring_t *coloring, 
					graph_t *graph, nv_t vertex, color_t new_color, char *to_do )
{
	
	color_t color1 = new_color, color2 = coloring->colors[vertex];
//	printf( "initial_vertex: %d,   color1 = %d    color2 = %d \n", vertex, color1, color2 );
	
	nv_t mark[coloring->num_vertices];
	nv_t i;
	
	g_breadth_first_search( graph, coloring, vertex, new_color, mark );
	
	for ( i=0; i<coloring->num_vertices; i++) {
		if (mark[i]==1) {
			if( coloring->colors[i] == color1 ) {
				coloring->colors[i] = color2;
				to_do[ i * coloring->num_colors + color2 ] = 0;
			} else if( coloring->colors[i] == color2 ) {
				coloring->colors[i] = color1;
				to_do[ i * coloring->num_colors + color1 ] = 0;
			} else {
				msg( LOG_ERR, "kempe_switch, you are trying to switch vertex with color that differs from color1 and color2" );		
			}
		}
	}
	
}


void perform_kempe_switching_on_coloring( graph_t *graph, graph_t *graph_of_colorings, char **dictionary, coloring_t *coloring, int degree ) {
	long i, j;	
	int k;
	char to_do[coloring->num_vertices*coloring->num_colors];
	for ( i = 0; i<coloring->num_vertices; i++ ) {
		for ( j = 0; j<coloring->num_colors; j++ ) {
			if( coloring->colors[i] == j ) 
				to_do[i*coloring->num_colors+j] = 0; //if vertex already has that color, no need to switch it
			else
				to_do[i*coloring->num_colors+j] = 1;
		}
	}
	
	for ( i = 0; i<degree; i++ ) {
		for ( j = 0; j<coloring->num_colors; j++ ) {
			to_do[i*coloring->num_colors+j] = 0; //vertex is in fixed part, no need to switch it
		}
	}

	coloring_t *coloring_after_switch = g_create_coloring( coloring->num_colors, coloring->num_vertices);
	code_properties_t properties = g_code_properties( coloring->num_colors, graph->num_vertices );

	char *code1, *code2;
	code1 = g_create_code( &properties );
	code2 = g_create_code( &properties );
	nv_t vertex1, vertex2;


	
	int count = 0;
	for ( i = 0; i<coloring->num_vertices; i++ ) {
		for ( j = 0; j<coloring->num_colors; j++ ) {
			if ( to_do[i*coloring->num_colors+j] ) {
				g_coloring_copy_colors( coloring, coloring_after_switch );
				g_kempe_switch( coloring_after_switch, graph, i, j, to_do );
				
				int inside_factor = 1;
				for ( k = 0; k < degree; k++ ) {
					if ( coloring_after_switch->colors[k] != coloring->colors[k] ) { 
						inside_factor = 0;
					}
				}
				
				if (inside_factor) {
					g_coloring_to_code( coloring, code1, &properties );
					vertex1 = g_dictionary_get_index(dictionary, code1, &properties, 0, graph_of_colorings->num_vertices);
	
					g_coloring_to_code( coloring_after_switch, code2, &properties );
					vertex2 = g_dictionary_get_index(dictionary, code2, &properties, 0, graph_of_colorings->num_vertices);
					
					if (vertex1==-1 || vertex2 == -1 ) {

						
						msg( LOG_ERR, "such code was not found" );
						printf( "%ld %ld\n",  vertex1, vertex2 );
						g_dump_code( code1, &properties );
						g_dump_code( code2, &properties );									
						exit(0);
					}
					
					g_add_edge( graph_of_colorings, vertex1, vertex2 );
					count++;	

				}
			}
		}
	}
	
	g_destroy_coloring( &coloring_after_switch );
	g_destroy_code( &code1 );
	g_destroy_code( &code2 );
	
}


	

	
	//zober farbenie coloring
		//sprav vsetky mozne kempe prepnutia (vysledna farbra K) a do grafu farbeni uloz hranu F-K
			//kempe prepnutie je definovane vrcholom a farbou
			//filtrovanie prepnuti
				//bool matica vrcholy * farby. Vymazavat z matice uz spravene prepnutia.
				




//changes coloring, updates to_do
int g_1vertex_switch( coloring_t *coloring, 
					graph_t *graph, nv_t vertex, color_t new_color )
{
	coloring->colors[vertex] = new_color;
	return g_is_colored_regularly( graph, coloring->colors );
}

void perform_1vertex_switching_on_coloring( graph_t *graph, graph_t *graph_of_colorings, char **dictionary, coloring_t *coloring ) {
	long i, j;	
	
	coloring_t *coloring_after_switch = g_create_coloring( coloring->num_colors, coloring->num_vertices);
	code_properties_t properties = g_code_properties( coloring->num_colors, graph->num_vertices );

	char *code1, *code2;
	code1 = g_create_code( &properties );
	code2 = g_create_code( &properties );
	nv_t vertex1, vertex2;
	
	int count = 0;
	for ( i = 0; i<coloring->num_vertices; i++ ) {
		for ( j = 0; j<coloring->num_colors; j++ ) {
			if ( coloring->colors[i] != j ) {
				g_coloring_copy_colors( coloring, coloring_after_switch );
				
				if ( g_1vertex_switch( coloring_after_switch, graph, i, j ) ) {
					g_coloring_to_code( coloring, code1, &properties );
					vertex1 = g_dictionary_get_index(dictionary, code1, &properties, 0, graph_of_colorings->num_vertices);
	
					g_coloring_to_code( coloring_after_switch, code2, &properties );
					vertex2 = g_dictionary_get_index(dictionary, code2, &properties, 0, graph_of_colorings->num_vertices);
					
					if (vertex1==-1 || vertex2 == -1 ) {

						
						msg( LOG_ERR, "such code was not found" );
						printf( "%ld %ld\n",  vertex1, vertex2 );
						g_dump_code( code1, &properties );
						g_dump_code( code2, &properties );									
						exit(0);
					}
					
					g_add_edge( graph_of_colorings, vertex1, vertex2 );
					count++;	

				}
			}
		}
	}
	
	g_destroy_coloring( &coloring_after_switch );
	g_destroy_code( &code1 );
	g_destroy_code( &code2 );
	
}











				
				
				
long g_read_number( char *buffer, int *pos ) {
	int p = *pos;
	int i;
	unsigned long result = 0;
	
	if (buffer[p] < 126) {
		*pos += 1;
		return buffer[p]-63;
	}
	
	if (buffer[p+1] < 126) {
		*pos += 4; i=1;
		while (1) {
			result += buffer[p+i];
			if (i==3) break;
			result *= 64;
			i++;
		}
		return result;
	}
	
	*pos += 8; i=2;
	while (1) {
		result += buffer[p+i];
		if (i==7) break;
		result *= 64;
		i++;
	}
	return result;

}


graph_t *g_create_from_string( char *buffer ) {
	char bits, is_edge=0;
	nv_t num_vertices, i, j;

	int pos=0;
	//read num_vertices
	num_vertices = g_read_number( buffer, &pos );
	graph_t *graph = g_create( num_vertices );
	
	i=0; j=1; bits=0;
	pos--;
	while( j<num_vertices ) {
		if( bits == 0) { 
			pos++;
			buffer[pos]-=63;
		}
		is_edge = (buffer[pos] & 32)>>5;
		if (is_edge) {
			g_add_edge( graph, i, j );
		}
		buffer[pos] <<= 1;
		bits = (bits+1)%6;
		i+=1;
		if (i==j) {
			i=0;
			j+=1;
		}
	}
	return graph;	
}


graph_t *g_create_from_file( FILE *file ) {
	char buffer[256];
	if ( fgets( buffer, 256, file ) ) {
		return g_create_from_string( buffer );
	} else
		return NULL;
}

void g_write_number_to_file( FILE *file, long nv ) {
	int i;
	char buffer[7];

	if (nv<63) {
		fputc( nv+63, file );
		buffer[0] = 0;
	} else if (nv < 258048) {
		fputc( 126, file );
		buffer[3] = 0;
		for (i = 2; i>=0; i--) {
			buffer[i] = nv % 64 + 63;
			nv /= 64;
		}
	} 
	/* you can never fit such large number into nv anyway... nasty hack
	else if (nv < 68719476735) {
		fputc( 126, file );	fputc( 126, file );
		buffer[6] = 0;
		for (i = 5; i>=0; i--) {
			buffer[i] = nv % 64 + 63;
			nv /= 64;
		}
	}*/

	fputs( buffer, file ); //if nv<63, then buffer is empty 
	
}

void g_put_bite( char bite, char *buffer, char *bits, FILE *file ) {
	if (bite!=0 && bite!=1) {
		msg( LOG_ERR, "g_put_bite: %d is not 1 nor 0",  bite);
	}
	
	*buffer <<= 1;
	*buffer += bite;
	*bits = *bits+1;
	
	if( *bits == 6) { 
		//buffer is filled up, let's write the char
		*bits = 0;
		fputc( *buffer + 63, file );
		*buffer = 0;
	}
}


void g_put_binary( nv_t number, char *buffer, char *bits, FILE *file, int k ) {
	int i;
	for ( i = 0; i < k; i++ ) {
		g_put_bite( (number>>(k-1))%2, buffer, bits, file );
		number<<=1;
		
	}
}

void g_write_to_file( graph_t *graph, FILE *file ) {
	nv_t i, j;
	char bits;

	g_write_number_to_file( file, graph->num_vertices ); 
		
	//write edges
	i = 0; j = 1; bits = 0;
	char char_buf = 0;
	
	while( j<graph->num_vertices ) {
		g_put_bite( g_is_edge( graph, i, j ), &char_buf, &bits, file );

		i+=1;
		if (i==j) {
			i=0;  j+=1;
		}
	}
	
	if (bits == 0) {
		//everything is stored.
	} else {
		//we need to store last char_buf, but first add some trailing zeroes
		char_buf <<= (6-bits);
		fputc( char_buf + 63, file );
	}
	fputc( '\n', file );
}




void g_write_to_file_sparse( graph_t *graph, FILE *file ) {
	//special case for empty graph
	if (graph->num_vertices==0) {
		fprintf( file, "?\n" );
		return;
	}

	fputc( ':', file ); //to distinguish the code from graph6 format

	g_write_number_to_file( file, graph->num_vertices ); 
		
	//k is the number of bits needed to represent n-1 in binary
	int k = 1 + floor( log10( graph->num_vertices-1 ) / log10( 2 ) ); 
	
	//v is current vertex
	nv_t i = 0, v = 0; 
	char char_buf = 0, bits = 0;
	
	nv_t last_vertex_with_an_edge = 0;
	
	//write edges
	while( v<graph->num_vertices ) {
		for ( i=0; i<v; i++ ) {			
			if ( g_is_edge( graph, i, v ) ) {
				//if the last written edge was incident with v,
				//then current b = 0
				if ( last_vertex_with_an_edge==v ) { 
					g_put_bite( 0, &char_buf, &bits, file );
				}
				
				//if the last written edge was incident with v-1,
				//then we need to increase v, so current b = 1		
				else if ( last_vertex_with_an_edge==v-1 ) { 
					g_put_bite( 1, &char_buf, &bits, file );
					last_vertex_with_an_edge=v;
				}

				//if the last written edge was incident with vertex with index
				//less then v-1, then jump is neccesarry
				else if ( last_vertex_with_an_edge < v-1 ) {
					g_put_bite( 1, &char_buf, &bits, file );
					g_put_binary( v, &char_buf, &bits, file, k );
					g_put_bite( 0, &char_buf, &bits, file );
					last_vertex_with_an_edge=v;
				}
				
				//current v is less then last_vertex_with_an_edge? timetravel?
				else {
					msg( LOG_ERR, "write to file sparse failed." );
				}

				g_put_binary( i, &char_buf, &bits, file, k );
			}
		}

		v+=1;
	}
	
	if (bits == 0) {
		//everything is stored.
	} else {
		//we need to store last char_buf, but first add some padding according
		//to the specification

		if ( 
			(
			((graph->num_vertices == 2) && (k == 1)) ||
			((graph->num_vertices == 4) && (k == 2)) ||
			((graph->num_vertices == 8) && (k == 4)) ||
			((graph->num_vertices == 16) && (k == 5))
			) 
			
			//at least k+1 bits to pad
			&& (6-bits >= k+1) 
			//vertex n-2 has an edge and vertex n-1 doesn't have an edge
			&& (last_vertex_with_an_edge==graph->num_vertices-2) 
			
			)
		{
			char_buf <<= 1;
		} else {
			char_buf <<= 1;
			char_buf += 1;
		}
		
		for (i = 0; i< 6-bits-1; i++ ) {
			char_buf <<= 1;
			char_buf += 1;
		}
		fputc( char_buf + 63, file );
	}
	fputc( '\n', file );
}


void g_dump_graphml_edgecolors( graph_t *graph, coloring_t *coloring, int id ) {
	printf("      <graph id=\"G%d\" edgedefault=\"undirected\"> \n ", id );
	nv_t i, j;
	
	for ( i=0; i<graph->num_vertices; i++ ) {
		printf("        <node id=\"G%d:n%lu\">\n", id, i );
		printf("          <data key=\"d0\">\n" );
		printf("            <y:ShapeNode>\n" );
		printf("              <y:Geometry width=\"10.0\" height=\"10.0\"/>\n" );
		printf("              <y:Fill color=\"#eeeeee\" transparent=\"false\"/>\n" );
		printf("              <y:NodeLabel>%lu</y:NodeLabel>\n", i );
		printf("              <y:Shape type=\"ellipse\"/>\n" );
		printf("            </y:ShapeNode>\n" );
		printf("          </data>\n" );
		printf("        </node>\n\n" );
	}

	char buffer[10];
	
	for ( i=0; i<graph->num_vertices-1; i++ ) {
		for ( j=i+1; j<graph->num_vertices; j++ ) {
			if ( g_is_edge( graph, i, j ) ) {
				int c = g_get_edge_color(  graph, coloring, i, j );
				printf("        <edge id=\"G%d:e%lu:%lu\" source=\"G%d:n%lu\" target=\"G%d:n%lu\">\n", id, i, j, id, i, id, j);
				printf("          <data key=\"d1\">                                            \n");
				printf("        	<y:PolyLineEdge>                                         \n");
				printf("        	  <y:LineStyle type=\"line\" width=\"4.0\" color=\"#%s\"/> \n", get_color_code( c, buffer ) ); 
				printf("        	  <y:EdgeLabel>%d</y:EdgeLabel>\n", c );
				printf("        	</y:PolyLineEdge>                                        \n");
				printf("          </data>                                                    \n");
				printf("        </edge>                                                      \n");
			}
		}
	}
	printf("      </graph> \n" );

}


void g_dump_goc_graphml( graph_t *graph,
						graph_t *graph_of_colorings,
						graph_t *linegraph, 
						char **dictionary,
						int num_colors ) {

	//print header
	printf( "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" );
	printf( "<graphml xmlns=\"http://graphml.graphdrawing.org/xmlns\"\n" );
	printf( "         xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n" );  
	printf( "         xsi:schemaLocation=\"http://graphml.graphdrawing.org/xmlns\n" );  
	printf( "         http://www.yworks.com/xml/schema/graphml/1.1/ygraphml.xsd\"\n" );  
	printf( "         xmlns:y=\"http://www.yworks.com/xml/graphml\">\n" );  
	printf( "  <key id=\"d0\" for=\"node\" yfiles.type=\"nodegraphics\"/>  \n" );
	printf( "  <key id=\"d1\" for=\"edge\" yfiles.type=\"edgegraphics\"/>  \n" );
	printf( "  <graph id=\"G\" edgedefault=\"undirected\"> \n" );

	
	code_properties_t properties = g_code_properties( num_colors, linegraph->num_vertices );
	coloring_t *coloring = g_create_coloring( num_colors, linegraph->num_vertices);
	
	//print nodes
	nv_t i;
	for ( i=0; i<graph_of_colorings->num_vertices; i++ ) {
		printf("    <node id=\"n%lu\" yfiles.foldertype=\"folder\">\n", i );
		printf("      <data key=\"d0\">                                   \n" );
		printf("        <y:GroupNode>                                   \n" );
		printf("          <y:NodeLabel>%lu</y:NodeLabel>\n \n", i );
		printf("          <y:Fill color=\"#eeeeee\" transparent=\"false\"/> \n" );
		printf("        </y:GroupNode>                                  \n" );
		printf("      </data>                                           \n" );

		g_erase_coloring( coloring );
		g_code_to_coloring( dictionary[i], coloring, &properties );

		//print a colored graph
		g_dump_graphml_edgecolors( graph, coloring, i );
		printf("    </node>\n" );
	}

	

	//print edges
	struct edge_t edge;
	for ( i=0; i<graph_of_colorings->num_vertices; i++ ) {
		edge = graph_of_colorings->vertices[i];
		if ( edge.vertex<0 ) continue;
		
		while( 1 ) {
			//print edge [a,b] only if a<b, so that there are not duplicate edges
			if (i<edge.vertex) {
				printf("    <edge id=\"e%lu:%lu\" source=\"n%lu\" target=\"n%lu\">\n", i, edge.vertex, i, edge.vertex);
				printf("      <data key=\"d1\">                                            \n");
				printf("    	<y:PolyLineEdge>                                         \n");
				printf("    	  <y:LineStyle type=\"line\" width=\"2.0\" color=\"#000000\"/> \n"); 
				printf("    	  <y:EdgeLabel></y:EdgeLabel>                            \n");
				printf("    	</y:PolyLineEdge>                                        \n");
				printf("      </data>                                                    \n");
				printf("    </edge>                                                      \n");
			}
			if (edge.next==NULL) break;
			edge = *(edge.next);
		};
		
	}
	printf( "\n" );

	
	printf("  </graph>\n" );
	printf("</graphml>\n" );
	
}



void g_dump_wilma( graph_t *graph_of_colorings ) {

	//print header
	printf( "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n" );
	printf( "<!DOCTYPE WilmaGraph SYSTEM \"WilmaGraph.dtd\">             \n" );
	printf( "<WilmaGraph>                                                \n" );
	printf( "<Cluster>                                                   \n" );
	printf( "<LayoutEngineType Name=\"ForceLayout\">                     \n" );
	printf( "<Property Key=\"Spring\" Value=\"5.0\"/>                    \n" );
	printf( "<Property Key=\"Origin\" Value=\"1.0\"/>                    \n" );
	printf( "<Property Key=\"Repulsion\" Value=\"1.0\"/>                 \n" );
	printf( "<Property Key=\"VelocityAttenuation\" Value=\"0.02\"/>      \n" );
	printf( "</LayoutEngineType>                                         \n" );

	//print nodes
	nv_t i;
	for ( i=0; i<graph_of_colorings->num_vertices; i++ ) {
		printf("    <Node ID=\"N%lu\" />\n", i );
	}

	

	//print edges
	struct edge_t edge;
	for ( i=0; i<graph_of_colorings->num_vertices; i++ ) {
		edge = graph_of_colorings->vertices[i];
		if ( edge.vertex<0 ) continue;
		
		while( 1 ) {
			//print edge [a,b] only if a<b, so that there are not duplicate edges
			if (i<edge.vertex) {
				printf("    <Edge EndID=\"N%lu\" StartID=\"N%lu\" />\n", i, edge.vertex);
			}
			if (edge.next==NULL) break;
			edge = *(edge.next);
		}
	}
	printf( "\n" );

	
	printf("  </Cluster>\n" );
	printf("</WilmaGraph>\n" );
	
}


void g_dump_data( graph_t *graph ) { 
	int i, j;
	for ( i = 0; i < graph->num_vertices-1; i++ ) {
		for ( j = i+1; j < graph->num_vertices; j++ ) {
			if( g_is_edge( graph, i, j ) ) {
				printf("%d, %d\t \t%d\n", i, j, g_get_data( graph, i, j ) );
			}
		}
	}
}




// ******************   ATTRIBUTES   ******************

nv_t get_root( nv_t *nodes, nv_t node ) {
	nv_t root = node, parent;
	while (nodes[root] != root) {
		root = nodes[root];
	}
	
	
	while (nodes[node] != node) {
		parent = nodes[node];
		nodes[node] = root;
		node = parent;
	}

	return root;
}


void join_nodes( nv_t *nodes, nv_t node1, nv_t node2 ) {
	node1 = get_root(nodes, node1);
	node2 = get_root(nodes, node2);
	
	nodes[node1] = node2;
}

nv_t g_num_components( graph_t *graph ) {
	
	nv_t *nodes = malloc( sizeof( nv_t[graph->num_vertices] ) );
	
	nv_t i, number = 0;

	//initialize structure
	for (i = 0; i<graph->num_vertices; i++) {
		nodes[i]=i;
	}
	
	edge_iterator_t iterator;
	edge_t *edge;
	
	//join nodes according to graph vertices connectivity
	for (i = 0; i<graph->num_vertices; i++) {
		iterator = g_new_edge_iterator( graph );
		iterator.vertex = i;

		while ( NULL != (edge = g_edge_iterator_next_for_vertex( &iterator ) ) ) {
			join_nodes( nodes, i, edge->vertex );
		}
	}

	//count nodes
	for (i = 0; i<graph->num_vertices; i++) {
		if (nodes[i]==i) {
			number++;
		}
	}

	free( nodes );
	return number;	
}


char g_bipartiteness( graph_t *graph ) {
	coloring_t *coloring = g_create_coloring( 2, graph->num_vertices );
	char is_bipartite = g_get_next_coloring( graph, coloring );  
	g_destroy_coloring( &coloring );
	return is_bipartite;
}


char g_regularity( graph_t *graph ) {
	if ( graph->num_vertices == 0 ) {
		msg(LOG_WARN, "Regularity of an empty graph? Yes, it's regular, but...");
		return 1;
	}

	nv_t i, degree, count;

	edge_iterator_t iterator;
	edge_t *edge;

	iterator = g_new_edge_iterator( graph );
	iterator.vertex = 0;

	//count degree of first vertex
	count = 0;
	while ( NULL != (edge = g_edge_iterator_next_for_vertex( &iterator ) ) ) {
		count++;
	}
	degree = count;
	
	//compare to degrees of other vertices
	for (i = 1; i<graph->num_vertices; i++) {
		count = 0;
		while ( NULL != (edge = g_edge_iterator_next_for_vertex( &iterator ) ) ) {
			count++;
		}
		if ( count != degree ) return 0;
	}

	return 1;
}


char num_vertices_with_degree( graph_t *graph, int degree ) {
	if ( graph->num_vertices == 0 ) {
		return 0;
	}

	nv_t i, count_edges, count_vertices = 0;

	//compare to degrees of other vertices
	for (i = 0; i<graph->num_vertices; i++) {
		count_edges = g_vertex_degree( graph, i );
		if ( count_edges == degree ) count_vertices++;
	}

	return count_vertices;
}


char num_vertices_with_maxdegree( graph_t *graph, int degree ) {
	if ( graph->num_vertices == 0 ) {
		return 0;
	}

	nv_t i, count_edges, count_vertices = 0;

	//compare to degrees of other vertices
	for (i = 0; i<graph->num_vertices; i++) {
		count_edges = g_vertex_degree( graph, i );
		if ( count_edges <= degree ) count_vertices++;
	}

	return count_vertices;
}




int g_max_degree( graph_t *graph ) {
	int max = 0, count;
	nv_t i;
	for (i = 0; i<graph->num_vertices; i++) {
		count = g_vertex_degree( graph, i );
		if (count>max) max = count;
	}

	return max;
}

int g_vertex_degree( graph_t *graph, nv_t vertex ) {
	edge_iterator_t iterator;
	edge_t *edge;

	iterator = g_new_edge_iterator( graph );
	iterator.vertex = vertex;

	int count = 0;
	while ( NULL != (edge = g_edge_iterator_next_for_vertex( &iterator ) ) ) {
		count++;
	}

	return count;
}

// ******************   FILE   ******************

long count_num_lines( char *filename ) {
	FILE *file;
	
	if ( NULL == ( file = fopen( filename, "r" ) ) ) {
		return 0;
	}
	
	long counter = 0;
	
	char line [ 128 ];
	while ( fgets ( line, sizeof line, file ) != NULL ) {
		//don't count header
		if (strncmp( "graph#", line, 6 ) == 0 ) continue; 
		counter++;
	}
	fclose ( file );
	return counter;
}



//this assumes the file is already open for reading
void skip_lines( long num_lines, FILE *file ) {
	long i;
	char tmp[256];
	//ignore first "graph_offset" lines of input file
	for ( i = 0; i < num_lines; i++ ) {
		fgets( tmp, 256, file );
	}
}


